@@ -2,8 +2,8 @@
 # this repository contains the full copyright notices and license terms.
 from sql import Null
 
-from trytond.model import Workflow, ModelView, ModelSQL, fields
-from trytond.pyson import Not, Equal, Eval, Or, Bool
+from trytond.model import Workflow, Model, ModelView, ModelSQL, fields, Check
+from trytond.pyson import Eval
 from trytond import backend
 from trytond.transaction import Transaction
 from trytond.pool import Pool
@@ -11,7 +11,7 @@ from trytond.pool import Pool
 __all__ = ['Inventory', 'InventoryLine']
 
 STATES = {
-    'readonly': Not(Equal(Eval('state'), 'draft')),
+    'readonly': Eval('state') != 'draft',
 }
 DEPENDS = ['state']
 
@@ -22,13 +22,11 @@ class Inventory(Workflow, ModelSQL, ModelView):
     location = fields.Many2One(
         'stock.location', 'Location', required=True,
         domain=[('type', '=', 'storage')], states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('lines', [0]))),
+            'readonly': (Eval('state') != 'draft') | Eval('lines', [0]),
             },
         depends=['state'])
     date = fields.Date('Date', required=True, states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('lines', [0]))),
+            'readonly': (Eval('state') != 'draft') | Eval('lines', [0]),
             },
         depends=['state'])
     lost_found = fields.Many2One(
@@ -39,8 +37,7 @@ class Inventory(Workflow, ModelSQL, ModelView):
         depends=DEPENDS)
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('lines', [0]))),
+            'readonly': (Eval('state') != 'draft') | Eval('lines', [0]),
             },
         depends=['state'])
     state = fields.Selection([
@@ -142,6 +139,17 @@ class Inventory(Workflow, ModelSQL, ModelView):
         Line = Pool().get("stock.inventory.line")
         Line.cancel_move([l for i in inventories for l in i.lines])
 
+    @classmethod
+    def create(cls, values):
+        inventories = super(Inventory, cls).create(values)
+        cls.complete_lines(inventories, fill=False)
+        return inventories
+
+    @classmethod
+    def write(cls, inventories, values):
+        super(Inventory, cls).write(inventories, values)
+        cls.complete_lines(inventories, fill=False)
+
     @classmethod
     def copy(cls, inventories, default=None):
         pool = Pool()
@@ -163,12 +171,17 @@ class Inventory(Workflow, ModelSQL, ModelView):
                     'inventory': new_inventory.id,
                     'moves': None,
                     })
-            cls.complete_lines([new_inventory])
+            cls.complete_lines([new_inventory], fill=False)
             new_inventories.append(new_inventory)
         return new_inventories
 
     @staticmethod
-    def complete_lines(inventories):
+    def grouping():
+        return ('product',)
+
+    @classmethod
+    @ModelView.button
+    def complete_lines(cls, inventories, fill=True):
         '''
         Complete or update the inventories
         '''
@@ -176,25 +189,26 @@ class Inventory(Workflow, ModelSQL, ModelView):
         Line = pool.get('stock.inventory.line')
         Product = pool.get('product.product')
 
+        grouping = cls.grouping()
         to_create = []
         for inventory in inventories:
             # Compute product quantities
+            if fill:
+                product_ids = None
+            else:
+                product_ids = [l.product.id for l in inventory.lines]
             with Transaction().set_context(stock_date_end=inventory.date):
-                pbl = Product.products_by_location([inventory.location.id])
+                pbl = Product.products_by_location(
+                    [inventory.location.id], product_ids=product_ids,
+                    grouping=grouping)
 
             # Index some data
-            product2uom = {}
             product2type = {}
             product2consumable = {}
             for product in Product.browse([line[1] for line in pbl]):
-                product2uom[product.id] = product.default_uom.id
                 product2type[product.id] = product.type
                 product2consumable[product.id] = product.consumable
 
-            product_qty = {}
-            for (location, product), quantity in pbl.iteritems():
-                product_qty[product] = (quantity, product2uom[product])
-
             # Update existing lines
             for line in inventory.lines:
                 if not (line.product.active and
@@ -202,26 +216,30 @@ class Inventory(Workflow, ModelSQL, ModelView):
                         and not line.product.consumable):
                     Line.delete([line])
                     continue
-                if line.product.id in product_qty:
-                    quantity, uom_id = product_qty.pop(line.product.id)
-                elif line.product.id in product2uom:
-                    quantity, uom_id = 0.0, product2uom[line.product.id]
+
+                key = (inventory.location.id,) + line.unique_key
+                if key in pbl:
+                    quantity = pbl.pop(key)
                 else:
-                    quantity, uom_id = 0.0, line.product.default_uom.id
-                values = line.update_values4complete(quantity, uom_id)
+                    quantity = 0.0
+                values = line.update_values4complete(quantity)
                 if values:
                     Line.write([line], values)
 
+            if not fill:
+                continue
             # Create lines if needed
-            for product_id in product_qty:
+            for key, quantity in pbl.iteritems():
+                product_id = key[grouping.index('product') + 1]
                 if (product2type[product_id] != 'goods'
                         or product2consumable[product_id]):
                     continue
-                quantity, uom_id = product_qty[product_id]
                 if not quantity:
                     continue
-                values = Line.create_values4complete(product_id, inventory,
-                    quantity, uom_id)
+
+                values = Line.create_values4complete(inventory, quantity)
+                for i, fname in enumerate(grouping, 1):
+                    values[fname] = key[i]
                 to_create.append(values)
         if to_create:
             Line.create(to_create)
@@ -251,9 +269,10 @@ class InventoryLine(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(InventoryLine, cls).__setup__()
+        t = cls.__table__()
         cls._sql_constraints += [
-            ('check_line_qty_pos',
-                'CHECK(quantity >= 0.0)', 'Line quantity must be positive.'),
+            ('check_line_qty_pos', Check(t, t.quantity >= 0),
+                'Line quantity must be positive.'),
             ]
         cls._order.insert(0, ('product', 'ASC'))
 
@@ -309,7 +328,13 @@ class InventoryLine(ModelSQL, ModelView):
 
     @property
     def unique_key(self):
-        return (self.product,)
+        key = []
+        for fname in self.inventory.grouping():
+            value = getattr(self, fname)
+            if isinstance(value, Model):
+                value = value.id
+            key.append(value)
+        return tuple(key)
 
     @classmethod
     def cancel_move(cls, lines):
@@ -348,31 +373,27 @@ class InventoryLine(ModelSQL, ModelView):
             origin=self,
             )
 
-    def update_values4complete(self, quantity, uom_id):
+    def update_values4complete(self, quantity):
         '''
         Return update values to complete inventory
         '''
         values = {}
         # if nothing changed, no update
-        if self.quantity == self.expected_quantity == quantity \
-                and self.uom.id == uom_id:
+        if self.quantity == self.expected_quantity == quantity:
             return values
         values['expected_quantity'] = quantity
-        values['uom'] = uom_id
         # update also quantity field if not edited
         if self.quantity == self.expected_quantity:
             values['quantity'] = max(quantity, 0.0)
         return values
 
     @classmethod
-    def create_values4complete(cls, product_id, inventory, quantity, uom_id):
+    def create_values4complete(cls, inventory, quantity):
         '''
         Return create values to complete inventory
         '''
         return {
             'inventory': inventory.id,
-            'product': product_id,
             'expected_quantity': quantity,
             'quantity': max(quantity, 0.0),
-            'uom': uom_id,
         }
