@@ -5,7 +5,7 @@ from decimal import Decimal
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateView, Button, StateAction
 from trytond import backend
-from trytond.pyson import Not, Bool, Eval, Equal, PYSONEncoder, Date, If
+from trytond.pyson import Eval, PYSONEncoder, Date, If
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.tools import grouped_slice
@@ -15,7 +15,7 @@ __all__ = ['Location', 'Party', 'ProductsByLocationsStart',
 __metaclass__ = PoolMeta
 
 STATES = {
-    'readonly': Not(Bool(Eval('active'))),
+    'readonly': ~Eval('active'),
 }
 DEPENDS = ['active']
 
@@ -30,8 +30,8 @@ class Location(ModelSQL, ModelView):
     active = fields.Boolean('Active', select=True)
     address = fields.Many2One("party.address", "Address",
         states={
-            'invisible': Not(Equal(Eval('type'), 'warehouse')),
-            'readonly': Not(Bool(Eval('active'))),
+            'invisible': Eval('type') != 'warehouse',
+            'readonly': ~Eval('active'),
             },
         depends=['type', 'active'])
     type = fields.Selection([
@@ -41,6 +41,7 @@ class Location(ModelSQL, ModelView):
         ('warehouse', 'Warehouse'),
         ('storage', 'Storage'),
         ('production', 'Production'),
+        ('drop', 'Drop'),
         ('view', 'View'),
         ], 'Location type', states=STATES, depends=DEPENDS)
     parent = fields.Many2One("stock.location", "Parent", select=True,
@@ -54,9 +55,9 @@ class Location(ModelSQL, ModelView):
     childs = fields.One2Many("stock.location", "parent", "Children")
     input_location = fields.Many2One(
         "stock.location", "Input", states={
-            'invisible': Not(Equal(Eval('type'), 'warehouse')),
-            'readonly': Not(Bool(Eval('active'))),
-            'required': Equal(Eval('type'), 'warehouse'),
+            'invisible': Eval('type') != 'warehouse',
+            'readonly': ~Eval('active'),
+            'required': Eval('type') == 'warehouse',
             },
         domain=[
             ('type', '=', 'storage'),
@@ -68,9 +69,9 @@ class Location(ModelSQL, ModelView):
         depends=['type', 'active', 'id'])
     output_location = fields.Many2One(
         "stock.location", "Output", states={
-            'invisible': Not(Equal(Eval('type'), 'warehouse')),
-            'readonly': Not(Bool(Eval('active'))),
-            'required': Equal(Eval('type'), 'warehouse'),
+            'invisible': Eval('type') != 'warehouse',
+            'readonly': ~Eval('active'),
+            'required': Eval('type') == 'warehouse',
         },
         domain=[
             ('type', '=', 'storage'),
@@ -80,16 +81,27 @@ class Location(ModelSQL, ModelView):
         depends=['type', 'active', 'id'])
     storage_location = fields.Many2One(
         "stock.location", "Storage", states={
-            'invisible': Not(Equal(Eval('type'), 'warehouse')),
-            'readonly': Not(Bool(Eval('active'))),
-            'required': Equal(Eval('type'), 'warehouse'),
+            'invisible': Eval('type') != 'warehouse',
+            'readonly': ~Eval('active'),
+            'required': Eval('type') == 'warehouse',
         },
         domain=[
-            ('type', '=', 'storage'),
+            ('type', 'in', ['storage', 'view']),
             ['OR',
                 ('parent', 'child_of', [Eval('id')]),
                 ('parent', '=', None)]],
         depends=['type', 'active', 'id'])
+    picking_location = fields.Many2One(
+        'stock.location', 'Picking', states={
+            'invisible': Eval('type') != 'warehouse',
+            'readonly': ~Eval('active'),
+            },
+        domain=[
+            ('type', '=', 'storage'),
+            ('parent', 'child_of', [Eval('storage_location', -1)]),
+            ],
+        depends=['type', 'active', 'storage_location'],
+        help='If empty the Storage is used')
     quantity = fields.Function(fields.Float('Quantity'), 'get_quantity')
     forecast_quantity = fields.Function(fields.Float('Forecast Quantity'),
             'get_quantity')
@@ -164,9 +176,12 @@ class Location(ModelSQL, ModelView):
         invalid_move_types = ['warehouse', 'view']
         Move = Pool().get('stock.move')
         if (self.type in invalid_move_types
-                and Move.search(['OR',
-                        ('to_location', '=', self.id),
-                        ('from_location', '=', self.id),
+                and Move.search([
+                        ['OR',
+                            ('to_location', '=', self.id),
+                            ('from_location', '=', self.id),
+                            ],
+                        ('state', 'not in', ['staging', 'draft']),
                         ])):
             self.raise_user_error('invalid_type_for_moves', (self.rec_name,))
 
@@ -324,9 +339,6 @@ class Location(ModelSQL, ModelView):
         if default is None:
             default = {}
 
-        default['left'] = 0
-        default['right'] = 0
-
         res = []
         for location in locations:
             if location.type == 'warehouse':
