@@ -4,15 +4,16 @@ import datetime
 import operator
 from decimal import Decimal
 from functools import partial
+from collections import OrderedDict
 
 from sql import Literal, Union, Column, Null
 from sql.aggregate import Sum
 from sql.conditionals import Coalesce
 from sql.operators import Concat
 
-from trytond.model import Workflow, Model, ModelView, ModelSQL, fields
+from trytond.model import Workflow, Model, ModelView, ModelSQL, fields, Check
 from trytond import backend
-from trytond.pyson import In, Eval, Not, Equal, If, Bool
+from trytond.pyson import Eval, If, Bool
 from trytond.tools import reduce_ids
 from trytond.transaction import Transaction
 from trytond.pool import Pool
@@ -22,9 +23,15 @@ from trytond.modules.product import price_digits
 __all__ = ['StockMixin', 'Move']
 
 STATES = {
-    'readonly': In(Eval('state'), ['cancel', 'assigned', 'done']),
+    'readonly': Eval('state').in_(['cancel', 'assigned', 'done']),
 }
 DEPENDS = ['state']
+LOCATION_DOMAIN = [
+    If(Eval('state').in_(['staging', 'draft']),
+        ('type', 'not in', ['warehouse']),
+        ('type', 'not in', ['warehouse', 'view'])),
+    ]
+LOCATION_DEPENDS = ['state']
 
 
 class StockMixin:
@@ -174,11 +181,11 @@ class Move(Workflow, ModelSQL, ModelView):
     internal_quantity = fields.Float('Internal Quantity', readonly=True,
         required=True)
     from_location = fields.Many2One("stock.location", "From Location",
-        select=True, required=True, states=STATES, depends=DEPENDS,
-        domain=[('type', 'not in', ('warehouse', 'view'))])
+        select=True, required=True, states=STATES,
+        depends=DEPENDS + LOCATION_DEPENDS, domain=LOCATION_DOMAIN)
     to_location = fields.Many2One("stock.location", "To Location", select=True,
-        required=True, states=STATES, depends=DEPENDS,
-        domain=[('type', 'not in', ('warehouse', 'view'))])
+        required=True, states=STATES,
+        depends=DEPENDS + LOCATION_DEPENDS, domain=LOCATION_DOMAIN)
     shipment = fields.Reference('Shipment', selection='get_shipment',
         readonly=True, select=True)
     origin = fields.Reference('Origin', selection='get_origin', select=True,
@@ -187,7 +194,7 @@ class Move(Workflow, ModelSQL, ModelView):
             },
         depends=['state'])
     planned_date = fields.Date("Planned Date", states={
-            'readonly': (In(Eval('state'), ['cancel', 'assigned', 'done'])
+            'readonly': (Eval('state').in_(['cancel', 'assigned', 'done'])
                 | Eval('shipment'))
             }, depends=['state', 'shipment'],
         select=True)
@@ -205,27 +212,27 @@ class Move(Workflow, ModelSQL, ModelView):
         ], 'State', select=True, readonly=True)
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             },
         domain=[
-            ('id', If(In('company', Eval('context', {})), '=', '!='),
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ],
         depends=['state'])
     unit_price = fields.Numeric('Unit Price', digits=price_digits,
         states={
-            'invisible': Not(Bool(Eval('unit_price_required'))),
+            'invisible': ~Eval('unit_price_required'),
             'required': Bool(Eval('unit_price_required')),
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             },
         depends=['unit_price_required', 'state'])
     cost_price = fields.Numeric('Cost Price', digits=price_digits,
         readonly=True)
     currency = fields.Many2One('currency.currency', 'Currency',
         states={
-            'invisible': Not(Bool(Eval('unit_price_required'))),
+            'invisible': ~Eval('unit_price_required'),
             'required': Bool(Eval('unit_price_required')),
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             },
         depends=['unit_price_required', 'state'])
     unit_price_required = fields.Function(
@@ -236,20 +243,20 @@ class Move(Workflow, ModelSQL, ModelView):
     def __setup__(cls):
         super(Move, cls).__setup__()
         cls._deny_modify_assigned = set(['product', 'uom', 'quantity',
-            'from_location', 'to_location', 'company', 'unit_price',
-            'currency'])
+            'from_location', 'to_location', 'company', 'currency'])
         cls._deny_modify_done_cancel = (cls._deny_modify_assigned |
             set(['planned_date', 'effective_date', 'state']))
         cls._allow_modify_closed_period = set()
 
+        t = cls.__table__()
         cls._sql_constraints += [
-            ('check_move_qty_pos',
-                'CHECK(quantity >= 0.0)', 'Move quantity must be positive'),
+            ('check_move_qty_pos', Check(t, t.quantity >= 0),
+                'Move quantity must be positive'),
             ('check_move_internal_qty_pos',
-                'CHECK(internal_quantity >= 0.0)',
+                Check(t, t.internal_quantity >= 0),
                 'Internal move quantity must be positive'),
             ('check_from_to_locations',
-                'CHECK(from_location != to_location)',
+                Check(t, t.from_location != t.to_location),
                 'Source and destination location must be different'),
             ]
         cls._order[0] = ('id', 'DESC')
@@ -281,9 +288,11 @@ class Move(Workflow, ModelSQL, ModelView):
         cls._buttons.update({
                 'cancel': {
                     'invisible': ~Eval('state').in_(['draft', 'assigned']),
+                    'readonly': Eval('shipment'),
                     },
                 'draft': {
                     'invisible': ~Eval('state').in_(['assigned']),
+                    'readonly': Eval('shipment'),
                     },
                 'assign': {
                     'invisible': ~Eval('state').in_(['assigned']),
@@ -547,7 +556,7 @@ class Move(Workflow, ModelSQL, ModelView):
             product_qty = product.quantity
         else:
             product_qty = product.template.quantity
-        product_qty = Decimal(str(product_qty))
+        product_qty = Decimal(str(max(product_qty, 0)))
         # convert wrt currency
         with Transaction().set_context(date=self.effective_date):
             unit_price = Currency.compute(self.currency, self.unit_price,
@@ -628,7 +637,7 @@ class Move(Workflow, ModelSQL, ModelView):
                 and self.from_location.type == 'storage'
                 and self.product.cost_price_method == 'average'):
             self._update_product_cost_price('out')
-        if not self.cost_price:
+        if self.cost_price is None:
             self.cost_price = self.product.cost_price
 
     @classmethod
@@ -767,7 +776,7 @@ class Move(Workflow, ModelSQL, ModelView):
                 product_ids=[m.product.id for m in moves],
                 grouping=grouping)
 
-        def get_key(location):
+        def get_key(move, location):
             key = (location.id,)
             for field in grouping:
                 value = getattr(move, field)
@@ -783,7 +792,8 @@ class Move(Workflow, ModelSQL, ModelView):
                     success = False
                 continue
             to_location = move.to_location
-            location_qties = {}
+            # Keep location order for pick_product
+            location_qties = OrderedDict()
             if with_childs:
                 childs = Location.search([
                         ('parent', 'child_of', [move.from_location.id]),
@@ -791,7 +801,7 @@ class Move(Workflow, ModelSQL, ModelView):
             else:
                 childs = [move.from_location]
             for location in childs:
-                key = get_key(location)
+                key = get_key(move, location)
                 if key in pbl:
                     location_qties[location] = Uom.compute_qty(
                         move.product.default_uom, pbl[key], move.uom,
@@ -827,7 +837,8 @@ class Move(Workflow, ModelSQL, ModelView):
                 qty_default_uom = Uom.compute_qty(move.uom, qty,
                         move.product.default_uom, round=False)
 
-                from_key, to_key = get_key(from_location), get_key(to_location)
+                from_key = get_key(move, from_location)
+                to_key = get_key(move, to_location)
                 pbl[from_key] = pbl.get(from_key, 0.0) - qty_default_uom
                 pbl[to_key] = pbl.get(to_key, 0.0) + qty_default_uom
         return success
@@ -882,7 +893,7 @@ class Move(Workflow, ModelSQL, ModelView):
                 raise ValueError('"%s" has no field "%s"' % (Move, field))
         assert grouping_filter is None or len(grouping_filter) == len(grouping)
 
-        move_rule_query = Rule.domain_get('stock.move')
+        move_rule_query = Rule.query_get('stock.move')
 
         PeriodCache = Period.get_cache(grouping)
         period = None
