@@ -13,7 +13,7 @@ from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.modules.company import CompanyReport
 from trytond.wizard import Wizard, StateTransition, StateView, Button
 from trytond import backend
-from trytond.pyson import Eval, Not, Equal, If, Or, And, Bool, In, Id
+from trytond.pyson import Eval, If, Id
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.tools import reduce_ids, grouped_slice
@@ -45,25 +45,26 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
             },
         depends=['state'])
     planned_date = fields.Date('Planned Date', states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             },
         domain=[
-            ('id', If(In('company', Eval('context', {})), '=', '!='),
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ],
         depends=['state'])
     reference = fields.Char("Reference", size=None, select=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     supplier = fields.Many2One('party.party', 'Supplier',
         states={
-            'readonly': And(Or(Not(Equal(Eval('state'), 'draft')),
-                    Bool(Eval('incoming_moves', [0]))), Bool(Eval('supplier'))),
+            'readonly': (((Eval('state') != 'draft')
+                    | Eval('incoming_moves', [0]))
+                & Eval('supplier')),
             }, required=True,
         depends=['state', 'supplier'])
     supplier_location = fields.Function(fields.Many2One('stock.location',
@@ -71,14 +72,14 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
         'on_change_with_supplier_location')
     contact_address = fields.Many2One('party.address', 'Contact Address',
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, domain=[('party', '=', Eval('supplier'))],
         depends=['state', 'supplier'])
     warehouse = fields.Many2One('stock.location', "Warehouse",
         required=True, domain=[('type', '=', 'warehouse')],
         states={
-            'readonly': Or(In(Eval('state'), ['cancel', 'done']),
-                Bool(Eval('incoming_moves', [0]))),
+            'readonly': (Eval('state').in_(['cancel', 'done'])
+                | Eval('incoming_moves', [0])),
             }, depends=['state'])
     warehouse_input = fields.Function(fields.Many2One('stock.location',
             'Warehouse Input'),
@@ -115,7 +116,7 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
                 ('company', '=', Eval('company')),
                 ],
             states={
-                'readonly': In(Eval('state'), ['draft', 'done', 'cancel']),
+                'readonly': Eval('state').in_(['draft', 'done', 'cancel']),
                 },
             depends=['state', 'warehouse', 'warehouse_input',
                 'warehouse_storage', 'company']),
@@ -449,7 +450,8 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
     @Workflow.transition('draft')
     def draft(cls, shipments):
         Move = Pool().get('stock.move')
-        Move.draft([m for s in shipments for m in s.incoming_moves])
+        Move.draft([m for s in shipments for m in s.incoming_moves
+                if m.state != 'staging'])
         Move.delete([m for s in shipments for m in s.inventory_moves
                 if m.state in ('draft', 'cancel')])
 
@@ -485,39 +487,36 @@ class ShipmentInReturn(Workflow, ModelSQL, ModelView):
         depends=['state'])
     planned_date = fields.Date('Planned Date',
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             },
         domain=[
-            ('id', If(In('company', Eval('context', {})), '=', '!='),
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ],
         depends=['state'])
     code = fields.Char("Code", size=None, select=True, readonly=True)
     reference = fields.Char("Reference", size=None, select=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     from_location = fields.Many2One('stock.location', "From Location",
         required=True, states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('moves', [0]))),
-            }, domain=[('type', '=', 'storage')],
+            'readonly': (Eval('state') != 'draft') | Eval('moves', [0]),
+            }, domain=[('type', 'in', ['storage', 'view'])],
         depends=['state'])
     to_location = fields.Many2One('stock.location', "To Location",
         required=True, states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('moves', [0]))),
+            'readonly': (Eval('state') != 'draft') | Eval('moves', [0]),
             }, domain=[('type', '=', 'supplier')],
         depends=['state'])
     moves = fields.One2Many('stock.move', 'shipment', 'Moves',
         states={
-            'readonly': And(Or(Not(Equal(Eval('state'), 'draft')),
-                    Not(Bool(Eval('from_location')))),
-                Bool(Eval('to_location'))),
+            'readonly': (((Eval('state') != 'draft') | ~Eval('from_location'))
+                & Eval('to_location')),
             },
         domain=[
             ('from_location', '=', Eval('from_location')),
@@ -694,7 +693,8 @@ class ShipmentInReturn(Workflow, ModelSQL, ModelView):
     @Workflow.transition('draft')
     def draft(cls, shipments):
         Move = Pool().get('stock.move')
-        Move.draft([m for s in shipments for m in s.moves])
+        Move.draft([m for s in shipments for m in s.moves
+                if m.state != 'staging'])
 
     @classmethod
     @ModelView.button
@@ -765,21 +765,21 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
         depends=['state'])
     planned_date = fields.Date('Planned Date',
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             },
         domain=[
-            ('id', If(In('company', Eval('context', {})), '=', '!='),
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ],
         depends=['state'])
     customer = fields.Many2One('party.party', 'Customer', required=True,
         states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('outgoing_moves', [0]))),
+            'readonly': ((Eval('state') != 'draft')
+                | Eval('outgoing_moves', [0])),
             },
         depends=['state'])
     customer_location = fields.Function(fields.Many2One('stock.location',
@@ -787,17 +787,17 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
     delivery_address = fields.Many2One('party.address',
         'Delivery Address', required=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, domain=[('party', '=', Eval('customer'))],
         depends=['state', 'customer'])
     reference = fields.Char("Reference", size=None, select=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     warehouse = fields.Many2One('stock.location', "Warehouse", required=True,
         states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('outgoing_moves', [0]))),
+            'readonly': ((Eval('state') != 'draft')
+                | Eval('outgoing_moves', [0])),
             }, domain=[('type', '=', 'warehouse')],
         depends=['state'])
     warehouse_storage = fields.Function(fields.Many2One('stock.location',
@@ -1041,7 +1041,8 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
     def draft(cls, shipments):
         Move = Pool().get('stock.move')
         Move.draft([m for s in shipments
-                for m in s.inventory_moves + s.outgoing_moves])
+                for m in s.inventory_moves + s.outgoing_moves
+                if m.state != 'staging'])
 
     @classmethod
     @ModelView.button
@@ -1070,8 +1071,9 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
         'Return inventory move for the outgoing move'
         pool = Pool()
         Move = pool.get('stock.move')
+        wrh = move.shipment.warehouse
         return Move(
-            from_location=move.shipment.warehouse.storage_location,
+            from_location=wrh.picking_location or wrh.storage_location,
             to_location=move.from_location,
             product=move.product,
             uom=move.uom,
@@ -1315,21 +1317,21 @@ class ShipmentOutReturn(Workflow, ModelSQL, ModelView):
         depends=['state'])
     planned_date = fields.Date('Planned Date',
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             },
         domain=[
-            ('id', If(In('company', Eval('context', {})), '=', '!='),
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ],
         depends=['state'])
     customer = fields.Many2One('party.party', 'Customer', required=True,
         states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('incoming_moves', [0]))),
+            'readonly': ((Eval('state') != 'draft')
+                | Eval('incoming_moves', [0])),
             },
         depends=['state'])
     customer_location = fields.Function(fields.Many2One('stock.location',
@@ -1337,17 +1339,17 @@ class ShipmentOutReturn(Workflow, ModelSQL, ModelView):
     delivery_address = fields.Many2One('party.address',
         'Delivery Address', required=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, domain=[('party', '=', Eval('customer'))],
         depends=['state', 'customer'])
     reference = fields.Char("Reference", size=None, select=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     warehouse = fields.Many2One('stock.location', "Warehouse", required=True,
         states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('incoming_moves', [0]))),
+            'readonly': ((Eval('state') != 'draft')
+                | Eval('incoming_moves', [0])),
             }, domain=[('type', '=', 'warehouse')],
         depends=['state'])
     warehouse_storage = fields.Function(fields.Many2One('stock.location',
@@ -1626,7 +1628,8 @@ class ShipmentOutReturn(Workflow, ModelSQL, ModelView):
     @Workflow.transition('draft')
     def draft(cls, shipments):
         Move = Pool().get('stock.move')
-        Move.draft([m for s in shipments for m in s.incoming_moves])
+        Move.draft([m for s in shipments for m in s.incoming_moves
+                if m.state != 'staging'])
         Move.delete([m for s in shipments for m in s.inventory_moves
                 if m.state in ('draft', 'cancel')])
 
@@ -1751,46 +1754,50 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
         depends=['state'])
     planned_date = fields.Date('Planned Date',
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             },
         domain=[
-            ('id', If(In('company', Eval('context', {})), '=', '!='),
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ],
         depends=['state'])
     code = fields.Char("Code", size=None, select=True, readonly=True)
     reference = fields.Char("Reference", size=None, select=True,
         states={
-            'readonly': Not(Equal(Eval('state'), 'draft')),
+            'readonly': Eval('state') != 'draft',
             }, depends=['state'])
     from_location = fields.Many2One('stock.location', "From Location",
         required=True, states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('moves', [0]))),
+            'readonly': (Eval('state') != 'draft') | Eval('moves', [0]),
             },
         domain=[
-            ('type', 'in', ['storage', 'lost_found']),
+            ('type', 'in', ['view', 'storage', 'lost_found']),
             ], depends=['state'])
     to_location = fields.Many2One('stock.location', "To Location",
         required=True, states={
-            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('moves', [0]))),
+            'readonly': (Eval('state') != 'draft') | Eval('moves', [0]),
             }, domain=[
-            ('type', 'in', ['storage', 'lost_found']),
+            ('type', 'in', ['view', 'storage', 'lost_found']),
             ], depends=['state'])
     moves = fields.One2Many('stock.move', 'shipment', 'Moves',
         states={
-            'readonly': ((Eval('state') != 'draft')
+            'readonly': (Eval('state').in_(['cancel', 'assigned', 'done'])
                 | ~Eval('from_location') | ~Eval('to_location')),
             },
         domain=[
-            ('from_location', 'child_of', [Eval('from_location', -1)],
-                'parent'),
-            ('to_location', '=', Eval('to_location')),
+            If(Eval('state') == 'draft', [
+                    ('from_location', '=', Eval('from_location')),
+                    ('to_location', '=', Eval('to_location')),
+                    ], [
+                    ('from_location', 'child_of', [Eval('from_location', -1)],
+                        'parent'),
+                    ('to_location', 'child_of', [Eval('to_location', -1)],
+                        'parent'),
+                    ]),
             ('company', '=', Eval('company')),
             ],
         depends=['state', 'from_location', 'to_location', 'planned_date',
@@ -1937,16 +1944,10 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
     @ModelView.button
     @Workflow.transition('draft')
     def draft(cls, shipments):
-        Move = Pool().get('stock.move')
-        Move.draft([m for s in shipments for m in s.moves])
-
-    @classmethod
-    @ModelView.button
-    @Workflow.transition('waiting')
-    def wait(cls, shipments):
         Move = Pool().get('stock.move')
         # First reset state to draft to allow update from and to location
-        Move.draft([m for s in shipments for m in s.moves])
+        Move.draft([m for s in shipments for m in s.moves
+                if m.state != 'staging'])
         for shipment in shipments:
             Move.write([m for m in shipment.moves
                     if m.state != 'done'], {
@@ -1955,6 +1956,20 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
                     'planned_date': shipment.planned_date,
                     })
 
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('waiting')
+    def wait(cls, shipments):
+        Move = Pool().get('stock.move')
+        Move.draft([m for s in shipments for m in s.moves])
+        moves = []
+        for shipment in shipments:
+            for move in shipment.moves:
+                if move.state != 'done':
+                    move.planned_date = shipment.planned_date
+                    moves.append(move)
+        Move.save(moves)
+
     @classmethod
     @Workflow.transition('assigned')
     def assign(cls, shipments):
